#ifndef ZINK_DEVICE_INFO_H
#define ZINK_DEVICE_INFO_H

#include "util/u_memory.h"

#include <vulkan/vulkan.h>

#if defined(__APPLE__)
// Source of MVK_VERSION
// Source of VK_EXTX_PORTABILITY_SUBSET_EXTENSION_NAME
#include "MoltenVK/vk_mvk_moltenvk.h"
#endif

struct zink_screen;

struct zink_device_info {
   uint32_t device_version;

   bool have_KHR_maintenance1;
   bool have_KHR_external_memory;
   bool have_KHR_external_memory_fd;
   bool have_KHR_vulkan_memory_model;
   bool have_EXT_conditional_rendering;
   bool have_EXT_transform_feedback;
   bool have_EXT_index_type_uint8;
   bool have_EXT_robustness2;
   bool have_EXT_vertex_attribute_divisor;
   bool have_EXT_calibrated_timestamps;
   bool have_EXT_custom_border_color;
   bool have_EXT_blend_operation_advanced;
   bool have_EXT_extended_dynamic_state;
   bool have_EXT_pipeline_creation_cache_control;
   bool have_EXT_shader_stencil_export;
#ifdef VK_EXTX_PORTABILITY_SUBSET_EXTENSION_NAME
   bool have_EXTX_portability_subset;
#endif
   bool have_EXT_4444_formats;
   bool have_vulkan11;
   bool have_vulkan12;

   VkPhysicalDeviceFeatures2 feats;
   VkPhysicalDeviceVulkan11Features feats11;
   VkPhysicalDeviceVulkan12Features feats12;

   VkPhysicalDeviceProperties props;
   VkPhysicalDeviceVulkan11Properties props11;
   VkPhysicalDeviceVulkan12Properties props12;

   VkPhysicalDeviceMemoryProperties mem_props;

   
   
   
   
   VkPhysicalDeviceConditionalRenderingFeaturesEXT cond_render_feats;
   VkPhysicalDeviceTransformFeedbackFeaturesEXT tf_feats;
   VkPhysicalDeviceTransformFeedbackPropertiesEXT tf_props;
   VkPhysicalDeviceIndexTypeUint8FeaturesEXT index_uint8_feats;
   VkPhysicalDeviceRobustness2FeaturesEXT rb2_feats;
   VkPhysicalDeviceRobustness2PropertiesEXT rb2_props;
   VkPhysicalDeviceVertexAttributeDivisorFeaturesEXT vdiv_feats;
   VkPhysicalDeviceVertexAttributeDivisorPropertiesEXT vdiv_props;
   
   VkPhysicalDeviceCustomBorderColorFeaturesEXT border_color_feats;
   VkPhysicalDeviceCustomBorderColorPropertiesEXT border_color_props;
   VkPhysicalDeviceBlendOperationAdvancedPropertiesEXT blend_props;
   VkPhysicalDeviceExtendedDynamicStateFeaturesEXT dynamic_state_feats;
   VkPhysicalDevicePipelineCreationCacheControlFeaturesEXT pipeline_cache_control_feats;
   
#ifdef VK_EXTX_PORTABILITY_SUBSET_EXTENSION_NAME
   VkPhysicalDevicePortabilitySubsetFeaturesEXTX portability_subset_extx_feats;
   VkPhysicalDevicePortabilitySubsetPropertiesEXTX portability_subset_extx_props;
#endif
   VkPhysicalDevice4444FormatsFeaturesEXT format_4444_feats;

    const char *extensions[17];
    uint32_t num_extensions;
};

bool
zink_get_physical_device_info(struct zink_screen *screen);

#endif
