#include "nir_to_spirv/nir_to_spirv.h"

#include "nir.h"
#include "nir_builder.h"
#include "nir_search.h"
#include "nir_search_helpers.h"

/* What follows is NIR algebraic transform code for the following 2
 * transforms:
 *    ('b2b32', 'a') => ('b2i32', 'a')
 *    ('b2b1', 'a') => ('i2b1', 'a')
 */


   static const nir_search_variable search0_0 = {
   { nir_search_value_variable, -1 },
   0, /* a */
   false,
   nir_type_invalid,
   NULL,
   {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15},
};
static const nir_search_expression search0 = {
   { nir_search_value_expression, 32 },
   false, false,
   -1, 0,
   nir_op_b2b32,
   { &search0_0.value },
   NULL,
};

   /* replace0_0 -> search0_0 in the cache */
static const nir_search_expression replace0 = {
   { nir_search_value_expression, 32 },
   false, false,
   -1, 0,
   nir_op_b2i32,
   { &search0_0.value },
   NULL,
};

   /* search1_0 -> search0_0 in the cache */
static const nir_search_expression search1 = {
   { nir_search_value_expression, 1 },
   false, false,
   -1, 0,
   nir_op_b2b1,
   { &search0_0.value },
   NULL,
};

   /* replace1_0 -> search0_0 in the cache */
static const nir_search_expression replace1 = {
   { nir_search_value_expression, 1 },
   false, false,
   -1, 0,
   nir_op_i2b1,
   { &search0_0.value },
   NULL,
};


static const struct transform zink_nir_lower_b2b_state2_xforms[] = {
  { &search0, &replace0.value, 0 },
};
static const struct transform zink_nir_lower_b2b_state3_xforms[] = {
  { &search1, &replace1.value, 0 },
};

static const struct per_op_table zink_nir_lower_b2b_table[nir_num_search_ops] = {
   [nir_op_b2b32] = {
      .filter = (uint16_t []) {
         0,
         0,
         0,
         0,
      },
      
      .num_filtered_states = 1,
      .table = (uint16_t []) {
      
         2,
      },
   },
   [nir_op_b2b1] = {
      .filter = (uint16_t []) {
         0,
         0,
         0,
         0,
      },
      
      .num_filtered_states = 1,
      .table = (uint16_t []) {
      
         3,
      },
   },
};

const struct transform *zink_nir_lower_b2b_transforms[] = {
   NULL,
   NULL,
   zink_nir_lower_b2b_state2_xforms,
   zink_nir_lower_b2b_state3_xforms,
};

const uint16_t zink_nir_lower_b2b_transform_counts[] = {
   0,
   0,
   (uint16_t)ARRAY_SIZE(zink_nir_lower_b2b_state2_xforms),
   (uint16_t)ARRAY_SIZE(zink_nir_lower_b2b_state3_xforms),
};

bool
zink_nir_lower_b2b(nir_shader *shader)
{
   bool progress = false;
   bool condition_flags[1];
   const nir_shader_compiler_options *options = shader->options;
   const shader_info *info = &shader->info;
   (void) options;
   (void) info;

   condition_flags[0] = true;

   nir_foreach_function(function, shader) {
      if (function->impl) {
         progress |= nir_algebraic_impl(function->impl, condition_flags,
                                        zink_nir_lower_b2b_transforms,
                                        zink_nir_lower_b2b_transform_counts,
                                        zink_nir_lower_b2b_table);
      }
   }

   return progress;
}

